@@ -15,6 +15,10 @@ from .types import (
     ZeroOrMany,
 )
 
+from .moose_model import (
+    MooseModel,
+)
+
 from .olap_table import (
     OlapConfig,
     OlapTable,
@@ -114,88 +118,80 @@ from .registry import (
 
 __all__ = [
     # Types
-    'BaseTypedResource',
-    'TypedMooseResource',
-    'Columns',
-    'T',
-    'U',
-    'T_none',
-    'U_none',
-    'ZeroOrMany',
-
+    "BaseTypedResource",
+    "TypedMooseResource",
+    "Columns",
+    "MooseModel",
+    "T",
+    "U",
+    "T_none",
+    "U_none",
+    "ZeroOrMany",
     # OLAP Tables
-    'OlapConfig',
-    'OlapTable',
-    'InsertOptions',
-
+    "OlapConfig",
+    "OlapTable",
+    "InsertOptions",
     # Streams
-    'StreamConfig',
-    'TransformConfig',
-    'ConsumerConfig',
-    'Stream',
-    'DeadLetterModel',
-    'DeadLetterQueue',
-    'SubjectLatest',
-    'SubjectVersion',
-    'SchemaById',
-    'KafkaSchemaConfig',
-
+    "StreamConfig",
+    "TransformConfig",
+    "ConsumerConfig",
+    "Stream",
+    "DeadLetterModel",
+    "DeadLetterQueue",
+    "SubjectLatest",
+    "SubjectVersion",
+    "SchemaById",
+    "KafkaSchemaConfig",
     # Ingestion
-    'IngestConfig',
-    'IngestConfigWithDestination',
-    'IngestPipelineConfig',
-    'IngestApi',
-    'IngestPipeline',
-
+    "IngestConfig",
+    "IngestConfigWithDestination",
+    "IngestPipelineConfig",
+    "IngestApi",
+    "IngestPipeline",
     # Consumption
-    'ApiConfig',
-    'Api',
-    'get_moose_base_url',
-    'set_moose_base_url',
+    "ApiConfig",
+    "Api",
+    "get_moose_base_url",
+    "set_moose_base_url",
     # Backward compatibility aliases (deprecated)
-    'ConsumptionApi',
-    'EgressConfig',
-
+    "ConsumptionApi",
+    "EgressConfig",
     # SQL
-    'SqlResource',
-    'View',
-    'MaterializedViewOptions',
-    'MaterializedView',
-
+    "SqlResource",
+    "View",
+    "MaterializedViewOptions",
+    "MaterializedView",
     # Workflow
-    'TaskContext',
-    'TaskConfig',
-    'Task',
-    'WorkflowConfig',
-    'Workflow',
-
+    "TaskContext",
+    "TaskConfig",
+    "Task",
+    "WorkflowConfig",
+    "Workflow",
     # Lifecycle
-    'LifeCycle',
-
+    "LifeCycle",
     # WebApp
-    'WebApp',
-    'WebAppConfig',
-    'WebAppMetadata',
-    'ApiUtil',
-    'get_moose_utils',
-    'get_moose_dependency',
-
+    "WebApp",
+    "WebAppConfig",
+    "WebAppMetadata",
+    "ApiUtil",
+    "get_moose_utils",
+    "get_moose_dependency",
     # Registry
-    'get_tables',
-    'get_table',
-    'get_streams',
-    'get_stream',
-    'get_ingest_apis',
-    'get_ingest_api',
-    'get_apis',
-    'get_api',
-    'get_sql_resources',
-    'get_sql_resource',
-    'get_workflows',
-    'get_workflow',
-    'get_web_apps',
-    'get_web_app',
+    "get_tables",
+    "get_table",
+    "get_streams",
+    "get_stream",
+    "get_ingest_apis",
+    "get_ingest_api",
+    "get_apis",
+    "get_api",
+    "get_sql_resources",
+    "get_sql_resource",
+    "get_workflows",
+    "get_workflow",
+    "get_web_apps",
+    "get_web_app",
     # Backward compatibility aliases (deprecated)
-    'get_consumption_apis',
-    'get_consumption_api',
+    "get_consumption_apis",
+    "get_consumption_api",
 ]
