@@ -8,7 +8,7 @@ import dataclasses
 import datetime
 import json
 from typing import Any, Optional, Callable, Union, Literal, Generic
-from pydantic import BaseModel, ConfigDict, AliasGenerator
+from pydantic import BaseModel, ConfigDict, AliasGenerator, Field
 from pydantic.alias_generators import to_camel
 from kafka import KafkaProducer
 
@@ -20,6 +20,24 @@ from ..config.runtime import config_registry, RuntimeKafkaConfig
 from ..commons import get_kafka_producer
 
 
+class SubjectLatest(BaseModel):
+    name: str = Field(serialization_alias="subjectLatest")
+
+
+class SubjectVersion(BaseModel):
+    subject: str
+    version: int
+
+
+class SchemaById(BaseModel):
+    id: int
+
+
+class KafkaSchemaConfig(BaseModel):
+    kind: Literal["JSON", "AVRO", "PROTOBUF"]
+    reference: Union[SubjectLatest, SubjectVersion, SchemaById]
+
+
 class StreamConfig(BaseModel):
     """Configuration for data streams (e.g., Redpanda topics).
 
@@ -41,6 +59,7 @@ class StreamConfig(BaseModel):
     default_dead_letter_queue: "Optional[DeadLetterQueue]" = None
     # allow DeadLetterQueue
     model_config = ConfigDict(arbitrary_types_allowed=True)
+    schema_config: Optional[KafkaSchemaConfig] = None
 
 
 class TransformConfig(BaseModel):
@@ -249,9 +268,9 @@ class Stream(TypedMooseResource, Generic[T]):
             return []
         return [b.strip() for b in broker_string.split(",") if b.strip()]
 
-    def _get_memoized_producer(self) -> tuple[KafkaProducer, Any]:
+    def _get_memoized_producer(self) -> tuple[KafkaProducer, RuntimeKafkaConfig]:
         """Create or reuse a KafkaProducer using runtime configuration."""
-        cfg = config_registry.get_kafka_config()
+        cfg: RuntimeKafkaConfig = config_registry.get_kafka_config()
         current_hash = self._create_kafka_config_hash(cfg)
 
         if self._memoized_producer is not None and self._kafka_config_hash == current_hash:
@@ -267,16 +286,16 @@ class Stream(TypedMooseResource, Generic[T]):
             finally:
                 self._memoized_producer = None
 
-        brokers = self._parse_brokers(getattr(cfg, "broker", "localhost:19092"))
+        brokers = self._parse_brokers(cfg.broker)
         if not brokers:
-            raise RuntimeError(f"No valid broker addresses found in: '{getattr(cfg, 'broker', '')}'")
+            raise RuntimeError(f"No valid broker addresses found in: '{cfg.broker}'")
 
         producer = get_kafka_producer(
             broker=brokers,
-            sasl_username=getattr(cfg, "sasl_username", None),
-            sasl_password=getattr(cfg, "sasl_password", None),
-            sasl_mechanism=getattr(cfg, "sasl_mechanism", None),
-            security_protocol=getattr(cfg, "security_protocol", None),
+            sasl_username=cfg.sasl_username,
+            sasl_password=cfg.sasl_password,
+            sasl_mechanism=cfg.sasl_mechanism,
+            security_protocol=cfg.security_protocol,
             value_serializer=lambda v: v.model_dump_json().encode('utf-8'),
             acks="all",
         )
@@ -300,7 +319,9 @@ class Stream(TypedMooseResource, Generic[T]):
     def send(self, values: ZeroOrMany[T]) -> None:
         """Send one or more records to this stream's Kafka topic.
 
-        Values are JSON-serialized using EnhancedJSONEncoder.
+        If `schema_registry` (JSON) is configured, resolve schema id and
+        send using Confluent wire format (0x00 + 4-byte schema id + JSON bytes).
+        Otherwise, values are JSON-serialized.
         """
         # Normalize inputs to a flat list of records
         filtered: list[T] = []
@@ -326,11 +347,42 @@ class Stream(TypedMooseResource, Generic[T]):
                 )
 
         producer, cfg = self._get_memoized_producer()
-        topic = self._build_full_topic_name(getattr(cfg, "namespace", None))
+        topic = self._build_full_topic_name(cfg.namespace)
 
-        for rec in filtered:
-            producer.send(topic, value=rec)
-        producer.flush()
+        sr = self.config.schema_config
+        if sr is not None:
+            if sr.kind != "JSON":
+                raise NotImplementedError("Currently JSON Schema is supported.")
+            try:
+                from confluent_kafka.schema_registry import SchemaRegistryClient
+                from confluent_kafka.schema_registry.json_schema import JSONSerializer
+            except Exception as e:
+                raise RuntimeError(
+                    "confluent-kafka[json,schemaregistry] is required for Schema Registry JSON"
+                ) from e
+
+            sr_url = cfg.schema_registry_url
+            if not sr_url:
+                raise RuntimeError("Schema Registry URL not configured")
+            client = SchemaRegistryClient({"url": sr_url})
+
+            if isinstance(sr.reference, SchemaById):
+                schema = client.get_schema(sr.reference.id)
+            elif isinstance(sr.reference, SubjectLatest):
+                schema = client.get_latest_version(sr.reference.name).schema
+            else:
+                schema = client.get_version(sr.reference.subject, sr.reference.version).schema
+
+            serializer = JSONSerializer(schema, client)
+
+            for rec in filtered:
+                value_bytes = serializer(rec.model_dump())
+                producer.send(topic, value=value_bytes)
+            producer.flush()
+        else:
+            for rec in filtered:
+                producer.send(topic, value=rec)
+            producer.flush()
 
 
 class DeadLetterModel(BaseModel, Generic[T]):
