@@ -167,6 +167,16 @@ class DistributedConfigDict(BaseEngineConfigDict):
     policy_name: Optional[str] = None
 
 
+class IcebergS3ConfigDict(BaseEngineConfigDict):
+    """Configuration for IcebergS3 engine."""
+    engine: Literal["IcebergS3"] = "IcebergS3"
+    path: str
+    format: str
+    aws_access_key_id: Optional[str] = None
+    aws_secret_access_key: Optional[str] = None
+    compression: Optional[str] = None
+
+
 # Discriminated union of all engine configurations
 EngineConfigDict = Union[
     MergeTreeConfigDict,
@@ -180,7 +190,8 @@ EngineConfigDict = Union[
     S3QueueConfigDict,
     S3ConfigDict,
     BufferConfigDict,
-    DistributedConfigDict
+    DistributedConfigDict,
+    IcebergS3ConfigDict
 ]
 
 
@@ -198,6 +209,7 @@ class TableConfig(BaseModel):
         metadata: Optional metadata for the table.
         life_cycle: Lifecycle management setting for the table.
         table_settings: Optional table-level settings that can be modified with ALTER TABLE MODIFY SETTING.
+        cluster: Optional cluster name for ON CLUSTER support in ClickHouse.
     """
     model_config = model_config
 
@@ -214,6 +226,7 @@ class TableConfig(BaseModel):
     indexes: list[OlapConfig.TableIndex] = []
     ttl: Optional[str] = None
     database: Optional[str] = None
+    cluster: Optional[str] = None
 
 
 class TopicConfig(BaseModel):
@@ -506,7 +519,7 @@ def _convert_engine_instance_to_config_dict(engine: "EngineConfig") -> EngineCon
     Returns:
         EngineConfigDict with engine-specific configuration
     """
-    from moose_lib.blocks import S3QueueEngine, S3Engine, BufferEngine, DistributedEngine
+    from moose_lib.blocks import S3QueueEngine, S3Engine, BufferEngine, DistributedEngine, IcebergS3Engine
 
     # Try S3Queue first
     if isinstance(engine, S3QueueEngine):
@@ -558,6 +571,16 @@ def _convert_engine_instance_to_config_dict(engine: "EngineConfig") -> EngineCon
             policy_name=engine.policy_name
         )
 
+    # Try IcebergS3
+    if isinstance(engine, IcebergS3Engine):
+        return IcebergS3ConfigDict(
+            path=engine.path,
+            format=engine.format,
+            aws_access_key_id=engine.aws_access_key_id,
+            aws_secret_access_key=engine.aws_secret_access_key,
+            compression=engine.compression
+        )
+
     # Try basic engines
     basic_config = _convert_basic_engine_instance(engine)
     if basic_config:
@@ -696,6 +719,7 @@ def to_infra_map() -> dict:
             indexes=table.config.indexes,
             ttl=table.config.ttl,
             database=table.config.database,
+            cluster=table.config.cluster,
         )
 
     for name, stream in get_streams().items():
